define(["dojo/_base/declare",
        "dojo/_base/lang",
         "dojo/_base/array",
		 "esri2/logging/Logger",
         "dojo/_base/xhr",
         "esri2/upgrade/CheckboxTree",
         "dojo/aspect",
         "dojo/promise/all",
         "esriMaps/managers/StorageManager",
         "./StorageAdapter",
         "esriMaps/adapters/LogAdapter",
         "esriMaps/managers/LogManager",
         "esriMaps/config",
         "esriMaps/utils/logger!",
         "./ContentStoreJsonRest"],
		
 function(declare, lang, array, Logger, xhr, CheckboxTree, aspect, all, StorageManager, StorageAdapter, LogAdapter, LogManager, esriMapsConfig, logger, ContentStoreJsonRest){
	

	var bisProvider = declare(null, {
		
		constructor: function(options){
					
			this.logPath= "cognos.upgrade.BISPROVIDER";
			Logger.getSingleton().addLogPath( this.logPath, dojo.config.LogLevel);
		
			options = options || {};
			declare.safeMixin(this.options, options);
            
            this.logAdapter = new LogAdapter({settings: { defaultLevel: "info"} });
            new LogManager({adapter: this.logAdapter}).initialize();
		},
		
		options: {},
		
		getProviderId: function () {
			return "Cognos";
		},
		
		getContentAsTree: function (handler, treeId, params){
			
			params = params || {};
			declare.safeMixin(this.options, params);
		
		 	var treeModel = new ContentStoreJsonRest(this.options);
		    var tree = new CheckboxTree({
				"class": "ContentTree",
				model: treeModel,
				showRoot : false,
		        getIconClass:   function( item, opened){
		    		return  treeModel.getIconClass(item, opened);
		    	} 
			}, treeId);
		    
		    handler.onSuccess (tree);
		},
		
		search4EMReports: function (handler, /*array*/ items) { 
		
			 var _searchPath = items[0].identifier;
			 
			 if(items[0].type!="report") {
				// search reports/folders under current folder/package
				_searchPath += "/*[permission('write')][@shown='true'][@objectClass='package' or @objectClass='folder' or @objectClass='report']";
			 }
			 xhr.get({
					url: dojo.config.em4cServer + "/ws/configInfo/searchOldEsriMapSpecs?",
					content: { 
						campassport: this.options.campassport,
						searchPath: encodeHTMLString(_searchPath),
                        mapPrefix: "Esri_"
					},
				    handleAs: "json"
				}).then(function(data){
                    handler.onSuccess (data);
				}, function(error){
					handler.onFailure (error);
				});
		},
        
        updateMapSpecs2Server: function (handler, item) {
			var _searchPath = item.identifier;
            var mapSpecs = [];
            array.forEach(item.oldMaps, function (map) {
                 if(!map.error && map.upgradedMapSpec && map.mapId) {
                    var specjson = lang.clone(map.upgradedMapSpec);
                    delete specjson.tool;
                    delete specjson.arcgis;
                    delete specjson.symbol;
                    delete specjson.extension;
                    mapSpecs.push({"mapId": map.mapId, "upgradedMapSpec": specjson});
                 }
            });
            if(mapSpecs.length === 0) {
                console.log("no updates");
                handler.onFailure (item);
                return;
            }
            mapSpecs = JSON.stringify(mapSpecs);
            var self = this;
			xhr.post({
				url: dojo.config.em4cServer + "/ws/configInfo/updateEsriMapSpecs?",
				content: { 
					campassport: this.options.campassport,
					searchPath: encodeHTMLString(_searchPath),
                    mapSpecs: encodeHTMLString(mapSpecs)
				},
				handleAs: "text",
				load: function(data){
			 		handler.onSuccess(item);
				},
				error: function(error){
					handler.onFailure (item);
				}
			});
		},
		
		upgradeMapSpecs: function (handler, item) {
	        var oldMaps = item.oldMaps,
                count = oldMaps.length,
                i = 0, hasError = false;
            var convertConfig = function () {
                if (i >= count) {
                    if(!hasError) {
                       handler.onSuccess(item);
                    }else{
                       handler.onFailure(item);
                    }
                    return;
                }   
                var map = oldMaps[i++],
                    mapSpec = lang.clone(map.mapSpec);
                    
                var storageAdapter = new StorageAdapter(),
                     storageManager = new StorageManager({adapter: storageAdapter});
                storageAdapter.initialize(mapSpec);
                storageManager.load().then(function (result) {
                    
                    map.upgradedMapSpec = result;
                    map.upgradedMapSpec.version = esriMapsConfig.version;
                    convertConfig();
                
                }, function (error){
                    map.error = error;
                    hasError = true;
                    convertConfig ();
                });
           }; 
           convertConfig();
		}
	});
	return bisProvider;
});