define(["dojo/_base/declare","dojo/aspect", "dojo/_base/lang", "../../logging/Logger"],
		function( declare, aspect, lang, MyLogger){

	return declare(null,{
		
		
		
		connectBisEventHandler: function(bisEventHandler /*BisEventHandler*/, logTarget /*String*/ ){
			// summary:
			// 		This is a callback wrapper that creates a standard callback syntax with all handlers,
			//		as well as inserts a BIS level logging.						
			// description:
			//      The method is primarily called with the BIS level Modules to connect the Implementation 
			//		to the Interface in a standardized way.
			// returns:
			//        A revised version of the handle with the standardized "onSuccess" and "onFailure" methods
			var newCallback = {
				logTarget: logTarget,
				onSuccess: function(args){
					if(typeof this.logTarget != "undefined"){
						MyLogger.getSingleton().debug(this.logTarget, "onSuccess called with " + arguments.length + " arguments.");
					}
				},
				onFailure: function(args){
					if(typeof this.logTarget != "undefined"){
						MyLogger.getSingleton().debug(this.logTarget, "onFailure called with " + arguments.length + " arguments.");
					}
				}
			};
			
			// This ensures that the callback function get the args from the original call, not the return from the middle man.
			// In short it allows are Messages through with out having to do much
			if(bisEventHandler){	
				
				var receiveArguments = true;
				aspect.after(newCallback, "onSuccess", lang.hitch(bisEventHandler.successContext, bisEventHandler.onSuccess), receiveArguments );
				aspect.after(newCallback, "onFailure", lang.hitch(bisEventHandler.failureContext, bisEventHandler.onFailure), receiveArguments );
			}
			return newCallback;
		}
		
	});
	
	
});